//
//   CubeShowWnd.cpp : Implementation file for class CCubeShowWnd
//
//  CubeShow Version 1.0 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : May 23, 2002
//
//  Description
//
//  The class that actually implements the window used in the screen saver
//  is class CDrawWnd. When the screen saver is invoked in thumbnail preview
//  mode CDrawWnd writes to a parent window, which is normally the Display
//  Properties dialog. However, in Full Screen mode and Test mode, the
//  Screen Saver has to supply the parent window itself. That's the purpose of
//  this class. In full screen mode, the Create() method of this class is called
//  to create a topmost full screen window to draw on. In test mode, the
//  CreateTest() method is called to create a resizable window. Once the window
//  is created, the parent class, CDrawWnd can be relied on to do most of the
//  interesting work behind actually drawing to the screen.
//
//  Much of the code in this source file was posted on CodeGuru by chensu
//  at the following URL: http://www.codeguru.com/misc/scrfrmwk.shtml.
//  CodeGuru allows unrestricted use of the code samples on their web
//  site, and is well worth a visit at http://www.codeguru.com.
//
#include "stdafx.h"
#include <regstr.h>
#include "CubeShow.h"
#include "drawwnd.h"
#include "CubeShowwnd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

CCubeShowWnd::CCubeShowWnd()
{
}

CCubeShowWnd::~CCubeShowWnd()
{
}

//
// This method is called when the screen saver starts in full screen mode.
// 
BOOL CCubeShowWnd::Create()
{
    return this->CDrawWnd::Create( WS_EX_TOPMOST, 
                                   ::AfxGetAppName(),
                                   WS_VISIBLE | WS_POPUP,
                                   CRect( 0, 0,
                                          ::GetSystemMetrics(SM_CXSCREEN),
                                          ::GetSystemMetrics(SM_CYSCREEN)),
                                   NULL );
}

//
// In test mode, we create a frame window with a border and a fixed size of 320x240.
//
BOOL CCubeShowWnd::CreateTest()
{
    return this->CDrawWnd::Create( 0, 
                                   ::AfxGetAppName(),
                                   WS_VISIBLE | WS_BORDER,
                                   CRect(0, 0, 320, 240 ),
                                   NULL );
}
//-----------------------------------------------------------------------------
BEGIN_MESSAGE_MAP(CCubeShowWnd, CDrawWnd)
    //{{AFX_MSG_MAP(CCubeShowWnd)
    ON_WM_SYSCOMMAND()
    ON_WM_CREATE()
    ON_WM_DESTROY()
    ON_WM_SETCURSOR()
    ON_WM_NCACTIVATE()
    ON_WM_ACTIVATE()
    ON_WM_ACTIVATEAPP()
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONDOWN()
    ON_WM_MBUTTONDOWN()
    ON_WM_RBUTTONDOWN()
    ON_WM_KEYDOWN()
    ON_WM_SYSKEYDOWN()
    ON_WM_CLOSE()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()


//
// Intercept the screen saver and window close commands, ignoring them.
//
void CCubeShowWnd::OnSysCommand(UINT nID, LPARAM lParam)
{
    if (nID == SC_SCREENSAVE || nID == SC_CLOSE)
        return;

    this->CDrawWnd::OnSysCommand(nID, lParam);
}

//
// Some miscellaneous setup stuff that needs to be done when the window
// is first created.
//
int CCubeShowWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    m_bIsDialogActive = FALSE;
    m_bReallyClose = FALSE;

    if (this->CDrawWnd::OnCreate(lpCreateStruct) == -1)
        return -1;

    // disable system keys: CTRL-ALT-Delete and ALT-Tab and the Windows key
    UINT nOldVal;
    ::SystemParametersInfo(SPI_SCREENSAVERRUNNING, 1 , &nOldVal, 0);
    //
    // I allow the mouse, cursor to wobble a little bit which means I need to
    // track the initial position. However, I found that if the DirectX code has
    // to switch screen resolutions, the cursor position is changed. So I simply 
    // set the position to -1, -1 to indicate that the initial position is currently
    // unknown. I'll read it later after the screen saver has started.
    m_ptInitCursorPos.x = -1;
    m_ptInitCursorPos.y = -1;
    m_dwInitTime = ::GetTickCount();
    m_dwMouseThreshold = 5;
    return 0;
}
//-----------------------------------------------------------------------------

void CCubeShowWnd::OnDestroy()
{
    // restore system keys: CTRL-ALT-Delete and ALT-Tab and the Windows key
    UINT nOldVal;
    ::SystemParametersInfo(SPI_SCREENSAVERRUNNING, 0, &nOldVal, 0);

    ::PostQuitMessage(0);

    this->CDrawWnd::OnDestroy();
}

//
// When we are in full screen mode, we want to make the cursor disappear. However, 
// if the user is typing in their password, we want to have the arrow appear. Everything
// changes when that password dialog appears.
//

BOOL CCubeShowWnd::OnSetCursor( CWnd * /* pWnd     */,
                                UINT   /* nHitTest */,
                                UINT   /* nMessage */)
{
    ::SetCursor( m_bIsDialogActive ? ::AfxGetApp()->LoadStandardCursor(IDC_ARROW) : NULL);

    return TRUE;
}


BOOL CCubeShowWnd::OnNcActivate( BOOL bActive )
{
    if ( !m_bIsDialogActive )
    {
        if ( !bActive )
            return FALSE;
    }
    return this->CDrawWnd::OnNcActivate(bActive);
}

//
// This message is sent when a window is being activated or deactivated. We respond
// to the deactivation message by closing up shop.
//
void CCubeShowWnd::OnActivate( UINT nState, CWnd *pWndOther, BOOL bMinimized )
{
    if ( !m_bIsDialogActive )
    {
        if ( nState == WA_INACTIVE )
            this->CloseSaverWindow( "OnActivate" );
    }

    this->CDrawWnd::OnActivate(nState, pWndOther, bMinimized);
}

//
// If the system wants to deactivate the app, it does so here. Like many of the other
// message handlers in this unit, you can see that the message is ignored if the password
// dialog is currently popped up.
//
void CCubeShowWnd::OnActivateApp( BOOL bActive, HTASK hTask )
{
    if ( !m_bIsDialogActive )
    {
        if ( !bActive )
            this->CloseSaverWindow( "OnActivate" );
    }
    
    this->CDrawWnd::OnActivateApp(bActive, hTask);
}
//
// If the mouse has moved past a certain threshold, the screen saver exits.
//
void CCubeShowWnd::OnMouseMove( UINT nFlags, CPoint point )
{
    if ( !m_bIsDialogActive )
    {
        if ( this->ExceedThreshold() )
             this->CloseSaverWindow( "Mouse moved" );
    }
    this->CDrawWnd::OnMouseMove( nFlags, point );
}
//
// Clicking a mouse button kills the screen saver. The next three
// message handlers all deal with that.
//
void CCubeShowWnd::OnLButtonDown( UINT nFlags, CPoint point )
{
    if ( !m_bIsDialogActive )
    {
        this->CloseSaverWindow( "Left Mouse Button Down" );
    }
    this->CDrawWnd::OnLButtonDown( nFlags, point );
}

void CCubeShowWnd::OnMButtonDown( UINT nFlags, CPoint point )
{
    if ( !m_bIsDialogActive )
    {
        this->CloseSaverWindow( "Middle Mouse Button Down" );
    }

    this->CDrawWnd::OnMButtonDown( nFlags, point );
}

void CCubeShowWnd::OnRButtonDown( UINT nFlags, CPoint point )
{
    if ( !m_bIsDialogActive )
    {
        this->CloseSaverWindow( "Right Mouse Button Down" );
    }
    this->CDrawWnd::OnRButtonDown( nFlags, point );
}

//
// Pressing a key, any key, closes the screen saver. The next two
// handlers deal with that.
//
void CCubeShowWnd::OnKeyDown( UINT nChar, UINT nRepCnt, UINT nFlags )
{
    if ( !m_bIsDialogActive )
    {
        this->CloseSaverWindow( "Key Down" );
    }
    this->CDrawWnd::OnKeyDown( nChar, nRepCnt, nFlags );
}

void CCubeShowWnd::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    if (!m_bIsDialogActive)
    {
        this->CloseSaverWindow( "Sys Key Down" );
    }

    this->CDrawWnd::OnSysKeyDown(nChar, nRepCnt, nFlags);
}


//
// This is the handler for the WM_CLOSE message. Note that right off
// the bat, I refuse to acknowledge the command unless the m_bReallyClose
// member is set. That gets set by a call to member CloseScreenSaver().
// If it turns out that I really should be closing, I first have to check
// to see if I need to ask for a password and validate that.
//
void CCubeShowWnd::OnClose()
{
    if ( !m_bReallyClose )
        return;

    if ( !m_bIsDialogActive )
    {
        BOOL bCanClose = TRUE;
        if ( ::GetTickCount() - m_dwInitTime > 0 )
        {
            m_bIsDialogActive = TRUE;
            bCanClose = this->VerifyPassword( this->GetSafeHwnd() );
            m_bIsDialogActive = FALSE;
            ::GetCursorPos( &m_ptInitCursorPos );
        }
        //
        // If the password failed, I restart the screen saver and return
        // without giving permission to close the window.
        //
        if ( !bCanClose )
        {
            m_Pause = NO_PAUSE;
            CreateGraphicsObject( m_dwExStyle );
            return;
        }
    }
    this->CDrawWnd::OnClose();
}

//
// If I decide it's time to close the screen saver, I call this method,
// which kicks off the process.
//
void CCubeShowWnd::CloseSaverWindow( const char *reason )
{
    Trace( "Screen saver window being closed because %s\n", reason );
    m_bReallyClose = TRUE;
    this->PostMessage(WM_CLOSE);
}
//
// This routine simply chekcs to see if the mouse has moved
// far enough to close the screensaver. Note that early on
// this will get called with point values of -1 and -1. Those
// values are place holders that tell me I need to get the 
// current position and save it off.
//
BOOL CCubeShowWnd::ExceedThreshold() 
{
    CPoint pt;
    ::GetCursorPos( &pt );
    if ( m_ptInitCursorPos.x == -1 )
    {
        m_ptInitCursorPos = pt;
        Trace( "Setting initial mouse position\n" );
        return FALSE;
    }
    CSize szDxy = pt - m_ptInitCursorPos;
    if ( szDxy.cx < 0 )
        szDxy.cx = -szDxy.cx;
    if ( szDxy.cy < 0 )
        szDxy.cy = -szDxy.cy;

    return szDxy.cx > static_cast<int>( m_dwMouseThreshold ) ||
           szDxy.cy > static_cast<int>( m_dwMouseThreshold );
}

//
// This is the routine that actually takes care of performing the password
// validation under Window 9X.
//
BOOL CCubeShowWnd::VerifyPassword(HWND hWndParent) 
{
    // On Windows NT, we return TRUE immediately. This lets the saver quit,
    // and the system manages passwords.
    // On Windows 9x, we call VerifyScreenSavePwd. This checks the appropriate
    // registry key and, if necessary, pops up a verify dialog.

    OSVERSIONINFO osv;
    osv.dwOSVersionInfoSize = sizeof( osv );
    ::GetVersionEx( &osv );
    if ( osv.dwPlatformId == VER_PLATFORM_WIN32_NT )
        return TRUE;

    BOOL bRes = TRUE;
    HINSTANCE hpwdcpl = ::LoadLibrary( _T("PASSWORD.CPL") );
    if ( hpwdcpl != NULL )
    {
        typedef BOOL (WINAPI *PVERIFYSCREENSAVEPWD)(HWND hWnd);
        PVERIFYSCREENSAVEPWD pVerifyScreenSavePwd =
            reinterpret_cast<PVERIFYSCREENSAVEPWD>
                ( ::GetProcAddress( hpwdcpl, "VerifyScreenSavePwd" ) );
        //
        // This funny pause code is here so that I can tell the DirectX code to
        // shut down. I can't properly display a dialog on the screen until the
        // DirectX surface is dead. So this is kind of ugly, but I don't have
        // a better way to request that the Screen Saver shut down and then wait
        // for it to happen.
        //
        Trace( "Requesting pause\n" );
        m_Pause = REQUEST_PAUSE;
        for ( int i = 0 ; i < 50 ; i++ )
        {
            Sleep( 100 );
            MSG msg;
            while ( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE ))
            {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            }
            if ( m_Pause == ACK_PAUSE )
                break;
        }
        delete pGraphics;
        pGraphics = 0;
        if (pVerifyScreenSavePwd != NULL)
        {
            Trace( "About to call VerifyScreenSavePwd\n" );
            bRes = pVerifyScreenSavePwd( hWndParent );
        }
        else
            TRACE( _T("Unable to get VerifyPwProc address. Aborting\n") );
        m_Pause = NO_PAUSE;
        ::FreeLibrary( hpwdcpl );
    }
    else
        TRACE( _T("Unable to load PASSWORD.CPL. Aborting\n") );
    return bRes;
}
