//
//   DXGraphics.cpp : Implementation file for class DXGraphics
//
//  CubeShow Version 1.0 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : May 23, 2002
//
//  Description
//
//  DXGraphics is the class that sets up all of the necessary parts of DirectX,
//  then draws the cube to the screen upon request. I learned most of what I
//  needed to write this code from the tutorials on Ken Paulson's web site at
//  http://www.drunkenhyena.com.
//
#include <stdafx.h>
#include "resource.h"

//
// The DXGraphics constructor doesn't do a lot of work - most of the initialization takes
// place in the create() member. Here all that happens is any critical data members are set
// to their default or initial values.
//
DXGraphics::DXGraphics( HINSTANCE app ) 
  : appInstance( app ), 
    graphWindow( NULL ), 
    graphics( NULL ), 
    device( NULL ), 
    cube( NULL )
{
    //
    // The texture array is an important part of this program. At any given time,
    // texture[ i ] should contain a valid texture that can be displayed on cube 
    // face i. There are six default textures stored as BMP files that are loaded
    // immediately upon construction. As images are loaded in from the user's
    // image cache directory, the picturePresent[i] flag is set to be true for
    // each one.
    //
    for ( int i = 0 ; i < 6 ; i++ )
    {
        texture[ i ] = NULL;
        picturePresent[ i ] = false;
    }
    scrWidth = 320;
    scrHeight = 240;
    scrWindowed = true;
    xRotStep = 0.0;
    yRotStep = 0.0;
    zRotStep = 0.0;
    forceUpright = false;
    textureLoaderThread = 0;
    faceToRotate = 0;
    killEvent = NULL;
    lowScreenResolution = false;
    startTime = 0;
    ErrorString = 0;
}

DXGraphics::~DXGraphics() 
{
    //
    // The thread that is busily loading textures needs to be killed in a friendly fashion. The
    // killEvent event is the way to do it. The thread will see the killEvent set as soon as it
    // finishes loading its next texture.
    //
    if ( killEvent != NULL )
    {
      Trace( "Setting kill event for texture loading thread\n" );
      SetEvent( killEvent );
    } else
        Trace( "Can't set kill event, handle is NULL!!!\n" );
    //
    // After setting the kill flag I'm willing to wait up to 10 seconds - a long time, but
    // not impossible. A slow CPU loading a massive JPG can use up seconds and seconds of
    // run time.
    //
    if ( textureLoaderThread )
    {
      DWORD exitCode;
      ::GetExitCodeThread( textureLoaderThread->m_hThread, &exitCode );
      if ( exitCode == STILL_ACTIVE ) {
        Trace( "Waiting for thread to exit\n" );
        DWORD result = ::WaitForSingleObject( this->textureLoaderThread->m_hThread, 10000 );
        if ( result == WAIT_TIMEOUT )
            Trace( "Trouble killing texture loading thread\n" );
        else
            Trace( "Texture loading thread shut down properly\n" );
      } else
        Trace( "Thread has already terminated, not going to wait\n" );
    }
    if ( killEvent != NULL )
      ::CloseHandle( killEvent );
    //
    // Be a good citizen and release all the textures in the queue. Not particularly important,
    // but it will avoid issuing all the memory leak messages if someone has the debug version
    // of DirectX installed.
    //
    while ( queue.size() )
    {
        IDirect3DTexture8 *texture = queue.front();
        texture->Release();
        queue.pop_front();
    }
    //
    // cleanup() will destroy all the DirectX objects.
    //
    cleanup();          
}

void DXGraphics::cleanup() 
{
    Trace( "Entering DXGraphics::cleanup\n" );
    //
    // Release the six textures that were in use when the program stopped.
    //
    for ( int i = 0 ; i < 6 ; i++ )
    {
        if ( texture[ i ] != NULL )
        {
            texture[ i ]->Release();
            texture[ i ] = NULL;
        }
    }
    //
    // Release the cube vertices
    //
    if(cube != NULL) {
        cube->Release();
        cube = NULL;
    }
    //
    // Release the DirectX device
    //
    if(device != NULL) {
        device->Release();
        device = NULL;
    }
    //
    // Release the DirectX object.
    //
    if(graphics != NULL) {
        graphics->Release();
        graphics = NULL;
    }
    Trace( "Leaving DXGraphics::cleanup\n" );
}

//
// After calling the constructur, any routine that wants to use the 
// DXGraphics object has to call this create() routine. This is where
// all the work is actually performed.
//
void DXGraphics::create()
{
    Trace( "Starting Direct3D create routine\n" );
    //
    // CreateDX is where the DirectX device is created. There are quite
    // a few things that can go wrong during the creationg process. If
    // createDX() returns a false value, create() bails out leaving this
    // object in an uninitialized state. 
    //
    if ( !createDX() )
        return;
    //
    // This code creates the vertices that define the cube.
    //
    createCube();
    //
    // This loads the initial six textures that are embedded as resources in this EXE.
    //
    loadTexture();
    //
    // Miscellaneous rendering setup
    //
    device->SetRenderState(D3DRS_LIGHTING,FALSE);
    device->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW); 
    //
    // More setup of the 3D environment
    //
    D3DXMATRIX proj;
    D3DXMatrixPerspectiveFovLH( &proj, D3DXToRadian(45.0f), (float)scrWidth/(float)scrHeight, 1.0f, 100.0f );
    device->SetTransform( D3DTS_PROJECTION, (D3DMATRIX*) &proj );
    device->SetVertexShader( vertexFormat );
    device->SetStreamSource( 0, cube, sizeof( vertex_t ) );
    //
    // Create the texture loading thread and the event I'll need to kill it.
    //
    killEvent = ::CreateEvent( NULL, FALSE, FALSE, "KillEvent" );
    textureLoaderThread = AfxBeginThread( TextureLoader, this, THREAD_PRIORITY_BELOW_NORMAL );
    //
    // The member variables shown here will be used to position the cube while Cubeshow
    // runs. x, y, and z keep track of the cube's position. makeRight is the option value 
    // that determines whether CubeShow is supposed to attempt to keep images upright.
    //
    startTime = timeGetTime();
    x = 0.0f;
    y = 0.0f;
    z = (zMax + zMin) / 2;
    makeRight = 0;
    Trace( "Done with Direct3D setup\n" );
}

//
// This is where it all happens! Each time iterate() is called, it paints a picture of the
// cube on the screen.
//
void DXGraphics::iterate() 
{
    //
    // The positioning and rotation of the cube are always determined only by the elapsed time.
    // This means that the cube will move across the screen and rotate at the same speed 
    // regardless of CPU speed or graphics card speed.
    //
    DWORD currentTime = timeGetTime();
    float runTime = (float)( currentTime - startTime );
    runTime /= 1000.0;
    //
    // DirectX drawing initialization
    //
    device->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_ARGB( 0xFF, 0, 0, 0 ), 0.0f, 0 );
    device->BeginScene();
        
    //
    // There are two matrices that are used to position the cube on the screen. The translation
    // matrix takes into account the x, y, and z coordinates on the screen. The rotation matrix
    // takes into account the yaw, pitch, and roll, which are the degrees of rotation about the
    // three axes. These two matrices are combined to create the worldMatrix, which is where the
    // camera watching the cube is positioned.
    //
    D3DXMATRIX worldMatrix;
    D3DXMATRIX translateMatrix; 
    D3DXMATRIX rotMatrix; 

    D3DXMatrixRotationYawPitchRoll( &rotMatrix, runTime * yRotStep,runTime * xRotStep,runTime * zRotStep);
    D3DXMatrixTranslation( &translateMatrix, x, y, z );
    D3DXMatrixMultiply( &worldMatrix, &rotMatrix, &translateMatrix );
    //
    // Once you have the world matrix set up, you apply it to your DirectX scene with the
    // SetTransform() function. The cube will now be drawn at our arbitrary position and
    // rotation, basically for the cost of just four or five lines of code!
    //
    device->SetTransform( D3DTS_WORLD, &worldMatrix);
    //
    // This loop is responsible for drawing the six faces of the cube. Each face consists of
    // two triangles in a strip, and they are actually rendered, along with their texture, by
    // the DrawPrimitive function.
    //
    bool clipped = false;
    for( int i = 0; i < 6; ++i )
    {

              D3DCLIPSTATUS8 clipStatus = { 0, 0xffffffff };
              device->SetClipStatus( &clipStatus );
              device->SetTexture(0, texture[ i ] );
              device->DrawPrimitive(D3DPT_TRIANGLESTRIP,i*4,2 );
              device->GetClipStatus( &clipStatus );
              device->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW); 
              //
              // The clipStatus stuff you see here is what I use to detect when the cube hits the
              // edge of the screen. All I do when that happens is reverse the sign on the
              // velocity vector in that direction. This would be a pretty difficult task to
              // do on my own, letting DirectX tell me when clipping occurs is infinitely better.
              //
              if ( ( clipStatus.ClipUnion & D3DCS_RIGHT && xTransStep > 0 ) || ( clipStatus.ClipUnion & D3DCS_LEFT && xTransStep < 0 ) )
              {
                  xTransStep *= -1;
              }
              if ( ( clipStatus.ClipUnion & D3DCS_TOP && yTransStep > 0 ) || ( clipStatus.ClipUnion & D3DCS_BOTTOM && yTransStep < 0 ) )
              {
                  yTransStep *= -1;
              }
              if ( clipStatus.ClipUnion & (D3DCS_RIGHT | D3DCS_LEFT | D3DCS_TOP | D3DCS_BOTTOM ) )
                  clipped = true;
    }
    //
    // z axis clipping is easier, I just have a zmin and zmax value, and I don't want to ever 
    // move outside of the range they define. Moving out of bounds on the Z axis is handled
    // using the technique seen immediately above - I just invert the sign on the z axis velocity when
    // the cube steps out of bounds.
    //
    if ( z < zMin && zTransStep < 0 ) zTransStep *= -1;
    if ( z > zMax && zTransStep > 0 ) zTransStep *= -1;
    //
    // Update the positions
    //
    x += xTransStep;
    y += yTransStep;
    z += zTransStep;//*delta;
    //
    // Here's where I call the routine that forces the pictures to be
    // upright. The actual calculation is done in pairs, because if
    // I figure out the orientation of one face I naturally know the
    // orientation of its opposite number. 
    //
    if ( !clipped && this->forceUpright )
    {
        makeRightSideUp( &worldMatrix, makeRight );
        makeRight += 2;
        makeRight %= 6;
    }
    //
    // Render the picture to the screen!
    //
    device->EndScene();
    device->Present( NULL, NULL, NULL, NULL );
    //
    // Here's where I rotate in new images as they arrive from the texture
    // loading thread.
    //
    CSingleLock lock( &queueLock );
    lock.Lock();
    if (  queue.size() > 0 )
    {
        texture[ faceToRotate ]->Release();
        texture[ faceToRotate ] = queue.front();
        picturePresent[ faceToRotate ] = true;
        queue.pop_front();
        if ( ++faceToRotate == 6 )
            faceToRotate = 0;
    }
}

//
// Making sure that a face is right side up is done in a straightforward, brute force manner.
// I call the DirectX function ProcessVertices, which applies the transformation and creates
// an output buffer. It's then a starightforward manner for the given face to find the vertex
// on the given face that has the maximum value along the y axis. When all is said and done 
// I have the highest and next highest vertex, which tells me which edge of the face should
// get the top of the photo. After I figure out which edge that is, I just fiddle with the 
// texture parameters on each vertex and get the image aligned. 
//

void DXGraphics::makeRightSideUp( D3DXMATRIX *matrix, int face )
{
    IDirect3DVertexBuffer8* temp;
    device->CreateVertexBuffer( sizeof( tvertex_t ) * 24 , 0, tvertexFormat, D3DPOOL_MANAGED, &temp );
    device->ProcessVertices( 0, 0, 24, temp, 0 );
    //
    // After processing, I have a vertex buffer in variable tverts which will be used to
    // determine who is up top.
    //
    tvertex_t* tverts;
    temp->Lock( 0, 0,(BYTE**) &tverts,0 );
    int highest = 4*face;
    int next = -1;
    float nextval = -1e10;
    for ( int i = 1 ; i < 4 ; i++ )
    {
        if ( tverts[ 4 * face + i ].y > tverts[ highest ].y )
        {
            next = highest;
            nextval = tverts[ next ].y;
            highest = 4*face + i;
        } else if ( tverts[ 4 * face + i ].y > nextval )
        {
            next = 4*face + i;
            nextval = tverts[ 4 * face + i ].y;
        }
    }
    temp->Unlock();
    temp->Release();
    temp = NULL;
    highest -= 4*face;
    next -= 4*face;
    vertex_t* verticies;
    //
    // The two highest vertices are in highest and next. Now process.
    //
    cube->Lock( 0, 0,(BYTE**) &verticies, 0 );

    switch ( 10 * highest + next )
    {
    case 2  :
    case 20 :     
                if ( picturePresent[ face ] )
                {
                  verticies[ 4*face + 0 ].tv = 1.0f;
                  verticies[ 4*face + 0 ].tu = 0.0f;
                  verticies[ 4*face + 1 ].tv = 0.0f;
                  verticies[ 4*face + 1 ].tu = 0.0f;
                  verticies[ 4*face + 2 ].tv = 1.0f;
                  verticies[ 4*face + 2 ].tu = 1.0f;
                  verticies[ 4*face + 3 ].tv = 0.0f;
                  verticies[ 4*face + 3 ].tu = 1.0f;
                }

                if ( picturePresent[ face + 1 ] )
                {
                  verticies[ 4*face + 7 ].tv = 1.0f;
                  verticies[ 4*face + 7 ].tu = 0.0f;
                  verticies[ 4*face + 6 ].tv = 0.0f;
                  verticies[ 4*face + 6 ].tu = 0.0f;
                  verticies[ 4*face + 5 ].tv = 1.0f;
                  verticies[ 4*face + 5 ].tu = 1.0f;
                  verticies[ 4*face + 4 ].tv = 0.0f;
                  verticies[ 4*face + 4 ].tu = 1.0f;
                }
                break;

    case 1  :
    case 10 :     
                if ( picturePresent[ face ] )
                {
                  verticies[ 4*face + 1 ].tv = 1.0f;
                  verticies[ 4*face + 1 ].tu = 0.0f;
                  verticies[ 4*face + 3 ].tv = 0.0f;
                  verticies[ 4*face + 3 ].tu = 0.0f;
                  verticies[ 4*face + 0 ].tv = 1.0f;
                  verticies[ 4*face + 0 ].tu = 1.0f;
                  verticies[ 4*face + 2 ].tv = 0.0f;
                  verticies[ 4*face + 2 ].tu = 1.0f;
                }
                if ( picturePresent[ face + 1 ] )
                {
                  verticies[ 4*face + 5 ].tv = 1.0f;
                  verticies[ 4*face + 5 ].tu = 0.0f;
                  verticies[ 4*face + 7 ].tv = 0.0f;
                  verticies[ 4*face + 7 ].tu = 0.0f;
                  verticies[ 4*face + 4 ].tv = 1.0f;
                  verticies[ 4*face + 4 ].tu = 1.0f;
                  verticies[ 4*face + 6 ].tv = 0.0f;
                  verticies[ 4*face + 6 ].tu = 1.0f;
                }
                break;
    case 23 :     
    case 32 :     
                if ( picturePresent[ face ] )
                {              
                  verticies[ 4*face + 2 ].tv = 1.0f;
                  verticies[ 4*face + 2 ].tu = 0.0f;
                  verticies[ 4*face + 0 ].tv = 0.0f;
                  verticies[ 4*face + 0 ].tu = 0.0f;
                  verticies[ 4*face + 3 ].tv = 1.0f;
                  verticies[ 4*face + 3 ].tu = 1.0f;
                  verticies[ 4*face + 1 ].tv = 0.0f;
                  verticies[ 4*face + 1 ].tu = 1.0f;
                }

                if ( picturePresent[ face + 1 ] )
                {
                  verticies[ 4*face + 6 ].tv = 1.0f;
                  verticies[ 4*face + 6 ].tu = 0.0f;
                  verticies[ 4*face + 4 ].tv = 0.0f;
                  verticies[ 4*face + 4 ].tu = 0.0f;
                  verticies[ 4*face + 7 ].tv = 1.0f;
                  verticies[ 4*face + 7 ].tu = 1.0f;
                  verticies[ 4*face + 5 ].tv = 0.0f;
                  verticies[ 4*face + 5 ].tu = 1.0f;
                }
                break;
    case 13 :     
    case 31 :     
                if ( picturePresent[ face ] )
                {
                  verticies[ 4*face + 3 ].tv = 1.0f;
                  verticies[ 4*face + 3 ].tu = 0.0f;
                  verticies[ 4*face + 2 ].tv = 0.0f;
                  verticies[ 4*face + 2 ].tu = 0.0f;
                  verticies[ 4*face + 1 ].tv = 1.0f;
                  verticies[ 4*face + 1 ].tu = 1.0f;
                  verticies[ 4*face + 0 ].tv = 0.0f;
                  verticies[ 4*face + 0 ].tu = 1.0f;
                }
                if ( picturePresent[ face + 1 ] )
                {
                  verticies[ 4*face + 4 ].tv = 1.0f;
                  verticies[ 4*face + 4 ].tu = 0.0f;
                  verticies[ 4*face + 5 ].tv = 0.0f;
                  verticies[ 4*face + 5 ].tu = 0.0f;
                  verticies[ 4*face + 6 ].tv = 1.0f;
                  verticies[ 4*face + 6 ].tu = 1.0f;
                  verticies[ 4*face + 7 ].tv = 0.0f;
                  verticies[ 4*face + 7 ].tu = 1.0f;
                }
                break;
    }
    cube->Unlock();
}
//
// This is a long routine, but it isn't particularly complicated. There is just a lot of work
// involved trying to find a set of parameters that will work with CreateDevice(). If we are
// in a windowed environment the decision making process is simple - we have to work with 
// the current resolution and color depth. But if we're in full screen mode, we can chose 
// between the current color depth, or some other possibilities. We try 15 bit, 16 bit, and
// 32 bit.
//
// If none of those options work, we can also try to use the software rendering engine instead
// of the HAL, but only if the INI file allows it. (Not by default.)
//

bool DXGraphics::createDX() 
{
    //
    // The docs tell you to always use D3D_SDK_VERSION for the argument to Direct3DCreate(),
    // but since I'm building with the 8.1 SDK, it would require all users to have 8.1. I want
    // to allow 8.0 as well, since Windows 95 users can't upgrade to 8.1. So I use the magic
    // number of 120 as my argument. Seems to work.
    //
    if ( graphics == NULL )
        graphics = Direct3DCreate8( 120 );
    if ( graphics == NULL )
    {
        ErrorString = "Unable to initialize Direct3D";
        return false;
    }
    //
    // The first choice is always the current adapter mode. If we are in full screen mode, we pick
    // up the width and height.
    //
    D3DDISPLAYMODE mode;
    graphics->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &mode );
    if ( scrWidth == -1 )
        scrWidth = mode.Width;
    if ( scrHeight == -1 )
        scrHeight = mode.Height;
    //
    // There is an option in the INI file that allows you to force the screen saver to use 640x480
    // resolution. If that option is set we force it to be the screen width and height.
    //
    if ( lowScreenResolution && !scrWindowed )
    {
        scrWidth = 640;
        scrHeight = 480;
    }
    //
    // This giant parameter block is what needs to be set up for the call to CreateDevice.
    // 
    D3DPRESENT_PARAMETERS params;
    memset( &params, 0, sizeof( params ) );
    params.Windowed = scrWindowed;
    params.BackBufferCount = 1;
    params.MultiSampleType = D3DMULTISAMPLE_NONE;
    params.SwapEffect = D3DSWAPEFFECT_DISCARD;
    params.hDeviceWindow = graphWindow;

    D3DDEVTYPE createDevice;
    //
    // This block figures out which resolution will work when in full screen mode.
    //
    if( !scrWindowed ) 
    {
        params.BackBufferHeight = scrHeight;
        params.BackBufferWidth = scrWidth;
        params.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
        params.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;
        createDevice = D3DDEVTYPE_HAL;
        //
        // The method getGoodFormat() tells me which resolution works - hopefully the current
        // resolution, but I'll try 15, 16, and 32 bit as well.
        //
        params.BackBufferFormat = getGoodFormat( mode.Format, D3DDEVTYPE_HAL, "D3DDEVTYPE_HAL" );
        //
        // If I couldn't find a good format with the HAL, I can still try with the reference
        // rasterizer, but only if the user has specified it in the INI file. Not really
        // recommended, because it's painfully slow on all but the most ripping CPUs.
        //
        if ( params.BackBufferFormat == D3DFMT_UNKNOWN )
            if ( AfxGetApp()->GetProfileInt( "Settings", "AllowSoftwareAcceleration", 0 ) != 0 )
            {
                createDevice = D3DDEVTYPE_REF;
                params.BackBufferFormat = getGoodFormat( mode.Format, D3DDEVTYPE_REF, "D3DDEVTYPE_REF" );
            }
        if ( params.BackBufferFormat == D3DFMT_UNKNOWN )
        {
          Trace( "Couldn't find a good full screen DirectX format, giving up\n" );
          graphics->Release();
          graphics = NULL;
          ErrorString = "CubeShow was unable to find a fullscreen configuration that supports Direct3D";
          return false;
        }
    } else {
        //
        // This block of code finds a good set of parameters when in windowed mode. I can't fool with
        // the resolution or color depth, so if the HAL doesn't work all I can hope is that the
        // user allows software acceleration.
        //
        createDevice = D3DDEVTYPE_HAL;
        params.BackBufferFormat = mode.Format; 
        HRESULT hr = graphics->CheckDeviceType( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, mode.Format, mode.Format, TRUE );
        Trace( "CheckDeviceType(windowed) - device=D3DDEVTYPE_HAL - check of currentFormat returns: %s\n", DXGetErrorString8( hr ) );
        if ( hr != S_OK )
        {
            if ( AfxGetApp()->GetProfileInt( "Settings", "AllowSoftwareAcceleration", 0 ) != 0 )
            {
                createDevice = D3DDEVTYPE_REF;
                hr = graphics->CheckDeviceType( D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, mode.Format, mode.Format, TRUE );
                Trace( "CheckDeviceType(windowed) - device=D3DDEVTYPE_REF - check of currentFormat returns: %s\n", DXGetErrorString8( hr ) );
            }
            if ( hr != S_OK)
            {
                Trace( "Couldn't find a good windowed DirectX format, giving up\n" );
                graphics->Release();
                graphics = NULL;
                ErrorString = "CubeShow was unable to find a windowed configuration that supports Direct3D";
                return false;
            }
        }
    }
    //
    // Okay, if all is well I can create the device at this point. 
    //
    if( device == NULL ) 
    {
        HRESULT hr = graphics->CreateDevice( D3DADAPTER_DEFAULT,
                                             D3DDEVTYPE_HAL,
                                             graphWindow,
                                             D3DCREATE_SOFTWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED,
                                             &params,
                                             &device );
        if ( hr != S_OK )
        {
            Trace( "CreateDevice(HAL) returned error code: %s\n", DXGetErrorString8( hr ) );
            static char buf[ 256 ];
            sprintf( buf, "DirectX error opening device: %s", DXGetErrorString8( hr ) );
            ErrorString = buf;
            
            if ( AfxGetApp()->GetProfileInt( "Settings", "AllowSoftwareAcceleration", 0 ) != 0 )
            {
                hr = graphics->CreateDevice( D3DADAPTER_DEFAULT,
                                             D3DDEVTYPE_REF,
                                             graphWindow,
                                             D3DCREATE_SOFTWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED,
                                             &params,
                                             &device );
                Trace( "CreateDevice(REF) returned %x\n", hr );
                if ( hr != S_OK ) 
                    strcat( buf, ". Software acceleration failed as well." );
            }
            if ( hr != S_OK )
            {
                Trace( "CreateDevice(REF) returned error code: %s\n", DXGetErrorString8( hr ) );
                Trace( "CreateDX() failed - CreateDevice() mode failed using current screen mode\n" );
                graphics->Release();
                graphics = NULL;
                return false;
            }
        }
    }
    else
    {
        device->Reset(&params);
    }
    //
    // A little bit more setup for the rendering engine.
    //
    device->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
    device->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    device->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1 );
    device->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
    device->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
    return true;
}

//
// Creating the cube is simply a matter of setting up a bunch of vertices.
// At the same time, each vertex gets texture coordinates.
//
void DXGraphics::createCube()
{
    vertex_t cubeVerts[] = 
    { 
        { -0.5f,  0.5f, -0.5f, 0.0f, 0.0f },
        {  0.5f,  0.5f, -0.5f, 0.0f, 0.0f },
        { -0.5f, -0.5f, -0.5f, 0.0f, 0.0f },
        {  0.5f, -0.5f, -0.5f, 0.0f, 0.0f },
        { -0.5f,  0.5f,  0.5f, 0.0f, 0.0f },
        {  0.5f,  0.5f,  0.5f, 0.0f, 0.0f },
        { -0.5f, -0.5f,  0.5f, 0.0f, 0.0f },
        {  0.5f, -0.5f,  0.5f, 0.0f, 0.0f },
    }; 
    device->CreateVertexBuffer( sizeof( vertex_t ) * 24,
                                D3DUSAGE_WRITEONLY,
                                vertexFormat,
                                D3DPOOL_MANAGED,
                                &cube );

    vertex_t* vertices;
    cube->Lock( 0, 0, (BYTE**)&vertices, 0 );
    int i = 0;

    // Front Face
    vertices[ i   ]    = cubeVerts[ 0 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 1 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 2 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 1.0f;
    vertices[ i   ]    = cubeVerts[ 3 ];
    vertices[ i   ].tv = 0.0f; 
    vertices[ i++ ].tu = 1.0f;
 
    // Back Face
    vertices[ i   ]    = cubeVerts[ 5 ];
    vertices[ i   ].tv = 1.0f; 
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 4 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 7 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 1.0f;
    vertices[ i   ]    = cubeVerts[ 6 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 1.0f;

    // Left Face
    vertices[ i   ]    = cubeVerts[ 4 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[0];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 6 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 1.0f;
    vertices[ i   ]    = cubeVerts[ 2 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 1.0f;

    // Right Face
    vertices[ i   ]    = cubeVerts[ 1 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 5 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 3 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 1.0f;
    vertices[ i   ]    = cubeVerts[ 7 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 1.0f;

    // Top Face
    vertices[ i   ]    = cubeVerts[ 4 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 5 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 0 ];
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 1.0f;
    vertices[ i   ]    = cubeVerts[ 1 ];
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 1.0f;
 
    // Bottom Face
    vertices[ i   ]    = cubeVerts[ 7 ];  
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 6 ]; 
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 0.0f;
    vertices[ i   ]    = cubeVerts[ 3 ]; 
    vertices[ i   ].tv = 1.0f;
    vertices[ i++ ].tu = 1.0f;
    vertices[ i   ]    = cubeVerts[ 2 ]; 
    vertices[ i   ].tv = 0.0f;
    vertices[ i++ ].tu = 1.0f;
 
    cube->Unlock();
}

//
// This routine loads the initial textures needed at startup time. Just
// to keep things interesting, I use smaller textures when we are in windowed
// mode.
//

void DXGraphics::loadTexture()
{
    HRESULT hr;
    if ( scrWindowed )
    {
      Trace( "Loading small textures\n" ); 
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE1_SMALL ), &texture[ 0 ] );
      if ( FAILED( hr ) )
        Trace( "Attempt to create texture from resource in loadTexture() returned error: %s\n", DXGetErrorString8( hr ) );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE2_SMALL ), &texture[ 1 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE3_SMALL ), &texture[ 2 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE4_SMALL ), &texture[ 3 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE5_SMALL ), &texture[ 4 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE6_SMALL ), &texture[ 5 ] );
    }
    else
    {
      Trace( "Loading full sized textures\n" ); 
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE1 ), &texture[ 0 ] );
      if ( FAILED( hr ) )
        Trace( "Attempt to create texture from resource in loadTexture() returned error: %s\n", DXGetErrorString8( hr ) );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE2 ), &texture[ 1 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE3 ), &texture[ 2 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE4 ), &texture[ 3 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE5 ), &texture[ 4 ] );
      hr = D3DXCreateTextureFromResource( device, GetModuleHandle( NULL ), MAKEINTRESOURCE( IDB_FACE6 ), &texture[ 5 ] );
    }
    Trace( "Done loading textures\n" );
}

//
// The texture loading thread runs at a lower priority. It attempts to load a new
// texture every five seconds by default. (Can be overridden in the INI file.)
// As new textures are loaded they are pushed into a deque that is read by the
// the iterate() routine.

UINT DXGraphics::TextureLoader( LPVOID lParam )
{
    DXGraphics *graphics = (DXGraphics *) lParam;
    boolean firstTime = true;
    Trace( "Entering texture loader thread\n" );
    int textureSize = AfxGetApp()->GetProfileInt( "Settings", "TextureSize", 256 );
    int imageCount = AfxGetApp()->GetProfileInt( "Photos", "Count", -1 );
    for ( int image = 0 ; ; image++ )
    {
        //
        // Normally I wait five seconds between load attempts, but while I am filling up the cube for 
        // the first time I go with no delay.
        //
        int wait = firstTime && image < 6 ? 1 : AfxGetApp()->GetProfileInt( "Settings", "TimeBetweenImages", 5000 );
        DWORD result = ::WaitForSingleObject( graphics->killEvent, wait );
        if ( result == WAIT_OBJECT_0 )
        {
            Trace( "Leaving texture loader thread, saw kill event\n" );
            return 0;
        } else
            if ( result == WAIT_ABANDONED || result == WAIT_FAILED )
            {
                Trace( "Leaving texture loader thread, wait abandoned or failed\n" );
                return 0;
            }
        CSingleLock lock( &graphics->queueLock );
        lock.Lock();
        if ( graphics->queue.size() < 4 )
        {
            lock.Unlock();
            CString imageName;
            if ( image < 6 && imageCount > 0 )
                imageName.Format( "Entry%d", image % imageCount );
            else
                imageName.Format( "Entry%d", image );
            CString fileName = AfxGetApp()->GetProfileString( "Photos", imageName );
            if ( fileName.GetLength() <= 0 )
            {
                //
                // If there are six or fewer images, no point in staying in the thread, we're done.
                //
                if ( image <= 6 )
                {
                    Trace( "Leaving texture loader thread, 0 length file name, image <= 6\n" );
                    return 0;
                } else {
                  firstTime = false;
                  image = -1;
                }
            } else {
                IDirect3DTexture8 *texture;
                Trace( "Loading texture %s\n", fileName );
                HRESULT hr = D3DXCreateTextureFromFileEx( graphics->device, 
                                                          fileName,
                                                          textureSize, textureSize,
                                                          D3DX_DEFAULT,
                                                          0, 
                                                          D3DFMT_UNKNOWN, 
                                                          D3DPOOL_MANAGED,
                                                          D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,
                                                          D3DX_FILTER_TRIANGLE | D3DX_FILTER_MIRROR,
                                                          0, 
                                                          NULL, 
                                                          NULL, 
                                                          &texture );
                lock.Lock();
                if ( hr == 0 )
                {
                    Trace( "Load texture succeeded\n" );
                    graphics->queue.push_back( texture );        
                } else 
                    Trace( "Failed to load texture %s, error = %s\n", fileName, DXGetErrorString8( hr ) );
            }
        } else
            lock.Unlock();
    }
}

bool DXGraphics::good()
{
  return graphics != NULL;
}

//
// This routine tries to find a good color depth that will work with the current format.
//
D3DFORMAT DXGraphics::getGoodFormat( D3DFORMAT currentFormat, D3DDEVTYPE device, const CString& name  )
{
   HRESULT hr = graphics->CheckDeviceType( D3DADAPTER_DEFAULT, device, currentFormat, currentFormat, FALSE );
   Trace( "getGoodFormat - device=%s - check of currentFormat returns: %s\n", name, DXGetErrorString8( hr ) );
   if ( hr == S_OK )
       return currentFormat;
   hr = graphics->CheckDeviceType( D3DADAPTER_DEFAULT, device, D3DFMT_X1R5G5B5, D3DFMT_X1R5G5B5, FALSE );
   Trace( "getGoodFormat - device=%s - check of 15 bit returns: %s\n", name, DXGetErrorString8( hr ) );
   if ( hr == S_OK )
       return D3DFMT_X1R5G5B5;
   hr = graphics->CheckDeviceType( D3DADAPTER_DEFAULT, device, D3DFMT_R5G6B5, D3DFMT_R5G6B5, FALSE );
   Trace( "getGoodFormat - device=%s - check of 16 bit returns: %s\n", name, DXGetErrorString8( hr ) );
   if ( hr == S_OK )
       return D3DFMT_R5G6B5;
   hr = graphics->CheckDeviceType( D3DADAPTER_DEFAULT, device, D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8, FALSE );
   Trace( "getGoodFormat - device=%s - check of 32 bit returns: %s\n", name, DXGetErrorString8( hr ) );
   if ( hr == S_OK )
       return D3DFMT_X8R8G8B8;
   return D3DFMT_UNKNOWN;
}
