//
//   DrawWnd.cpp : Implementation file for class CDrawWnd
//
//  CubeShow Version 1.0 Copyright (c) 2002
//  Ziff Davis Media, Inc.
//  All rights reserved.
//  First Published in PC Magazine, US Edition, July 2000
//
//  Author : Mark Nelson
//  Date   : May 23, 2002
//
//  Description
//
//  This class works hand in hand with class CCubeShowWnd to display the screen saver.
//  CCubeShowWnd provides most of the behavior for the class, handling key presses,
//  mouse movement, and messages from the O/S. This is the base class, and it basically
//  only does a few important things: it creates the window in one of two different
//  ways, and actually does the cube drawing on a timer.
//

#include "stdafx.h"
#include "CubeShow.h"
#include "drawwnd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


//
// I think the AutoDelete argument is a waste of time, it will always be set
// to true. The m_Pause member is used to coordinate things when a password check is required.
//
CDrawWnd::CDrawWnd(BOOL bAutoDelete /* = TRUE */)
{
    m_bAutoDelete = bAutoDelete;
    m_Pause = NO_PAUSE;
    mmTimerId = NULL;
}

//
// pGraphics is the DirectX object that does all the work. It has to be deleted when
// the object is destroyed.
//
CDrawWnd::~CDrawWnd()
{
    if ( mmTimerId != NULL )
        timeKillEvent( mmTimerId );
    delete pGraphics;
    pGraphics = 0;
}

//
// This is the Create function that is called from the CCubeShowWnd. It is called with two 
// very distinct set of arguments, one which creates a full screen topmost window, and 
// another which creates a child window.
//
BOOL CDrawWnd::Create( DWORD dwExStyle, 
                       LPCTSTR lpszWindowName, 
                       DWORD dwStyle,
                       const RECT &rect, 
                       CWnd *pWndParent, 
                       UINT nID /* = 0 */ )
{
    BOOL result =  this->CreateEx( dwExStyle,
                                   ::AfxRegisterWndClass( CS_HREDRAW | CS_VREDRAW,
                                                          NULL,
                                                          static_cast<HBRUSH>( ::GetStockObject( BLACK_BRUSH ) ),
                                                          ::AfxGetApp()->LoadIcon( IDR_MAINFRAME ) ),
                                   lpszWindowName, 
                                   dwStyle,
                                   rect, 
                                   pWndParent, 
                                   nID );
    m_dwExStyle = dwExStyle;
    if ( result )
        CreateGraphicsObject( dwExStyle );
    return result;
}

//
// This class uses the slightly unconventional "delete this" technique seen so often in
// Windows C++ programs. An experienced C++ programmer seeing a window object destroyed
// in its own member function might find it odd, but it is safe in this particular 
// instance. 
//
void CDrawWnd::PostNcDestroy()
{
    if (m_bAutoDelete)
        delete this;
}

//-----------------------------------------------------------------------------
BEGIN_MESSAGE_MAP(CDrawWnd, CWnd)
    //{{AFX_MSG_MAP(CDrawWnd)
    ON_WM_PAINT()
    ON_WM_TIMER()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()
//-----------------------------------------------------------------------------

//
// The Paint routine is only used if we failed to create a good DirectX graphics
// object. In that case, the black screen gets one of two different messages, 
// depending on whether the window is the full screen mode or preview mode.
//
void CDrawWnd::OnPaint()
{
    CPaintDC dc( this );
    if ( !pGraphics->good() )
    {
        CFont font;
        font.CreateFont( 0, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, NULL );
        CDC *dc = GetDC();
        dc->SetTextColor( RGB( 0, 255, 255 ) );
        dc->SetBkColor( 0 );
        CString msg;
        if ( pGraphics->scrWindowed == 0 )
        {
            if ( pGraphics->error() == 0 )
                msg = "CubeShow was unable to find a display configuration that supports Direct3D";
            else
                msg = pGraphics->error();
        } else
            msg = "No thumbnail preview available";
        CSize size = dc->GetTextExtent( msg );
        CRect rect;
        GetClientRect( rect );
        CSize sz = dc->GetTextExtent( msg );
        CRect trect( 0, (rect.Height() - sz.cy) / 3, rect.Width(), rect.Height() );
        dc->DrawText( msg, trect, DT_CENTER | DT_WORDBREAK );
    }
}


//
// This is where we create and initialize the DirectX graphics object.
// As you can see from the code here, DirectX initialization takes a lot of
// setup. Once all the creation is done, a timer is set up to fire every 
// 20 milliseconds, and away we go.
//
void CDrawWnd::CreateGraphicsObject( DWORD dwExStyle )
{
    pGraphics = new DXGraphics ( ::AfxGetInstanceHandle() );
    pGraphics->graphWindow = m_hWnd;
    CRect clientRect;
    GetClientRect( &clientRect );
    pGraphics->scrWindowed = (dwExStyle & WS_EX_TOPMOST) == 0;
    pGraphics->scrHeight = clientRect.Height();
    pGraphics->scrWidth = clientRect.Width();
    pGraphics->setTranslations( AfxGetApp()->GetProfileInt( "Settings", "SpeedX", 15 ),
                                AfxGetApp()->GetProfileInt( "Settings", "SpeedY", 5 ), 
                                AfxGetApp()->GetProfileInt( "Settings", "SpeedZ", 0) );
    pGraphics->setRotations( AfxGetApp()->GetProfileInt( "Settings", "SpinX", 10 ), 
                             AfxGetApp()->GetProfileInt( "Settings", "SpinY", 15 ), 
                             AfxGetApp()->GetProfileInt( "Settings", "SpinZ", 0 ) );
    pGraphics->setSize( AfxGetApp()->GetProfileInt( "Settings", "CubeSize", 0  ) );
    pGraphics->setForceUpright( AfxGetApp()->GetProfileInt( "Settings", "KeepUpright", 0 ) ? true : false );
    pGraphics->setLowScreenResolution( AfxGetApp()->GetProfileInt( "Settings", "LowScreenResolution", 0 ) ? true : false );
    pGraphics->create();
    startTime = GetTickCount();
    count = 0;
    //
    // This call to SetTimer() meant that I was limited to the 18.2 ms system clock resolution
    // on Win9x systems. In the 1.1 release, I am using the multimedia timer, which should let
    // me fire off the timer on the 50 ms tick that I desire.
    //
    // UINT timer = SetTimer( 101, 20, 0 );
    //
    mmTimerId = timeSetEvent( 20,
                              5,
                              mmTimerProc,
                              (DWORD) this,
                              TIME_ONESHOT );
    if ( mmTimerId == NULL )
        UINT timer = SetTimer( 101, 20, 0 );
}

//
// The timer routine is where all the work really happens. Normally, I just call
// the iterate() method of the graphics object, which draws a new cube on the
// screen. However, if the user needs to enter a password, I can also go into
// pause mode, which means the graphics object stops rendering for a bit.
//
void CDrawWnd::OnTimer(UINT nIDEvent) 
{
    if ( !TimerWorker() ) {
        Trace( "Killing Windows Timer!\n" );
        KillTimer( nIDEvent );
        Trace( "Done killing Windows Timer\n" );
    }
}

void _stdcall CDrawWnd::mmTimerProc(UINT uTimerID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
    CDrawWnd *obj = (CDrawWnd *) dwUser;    
    if ( !obj->TimerWorker() ) {
        Trace( "Killing mmtimer!\n" );
        timeKillEvent( uTimerID );
        Trace( "Done killing mmtimer\n" );
    } else
        obj->mmTimerId = timeSetEvent( 20,
                                       5,
                                       mmTimerProc,
                                       (DWORD) obj,
                                       TIME_ONESHOT );
}

bool CDrawWnd::TimerWorker()
{
    if ( m_Pause == REQUEST_PAUSE )
    {
        Trace( "Acking pause\n" );
        m_Pause = ACK_PAUSE;
        return true;
    }
    if ( m_Pause == ACK_PAUSE || !pGraphics->good() )
        return false;
    //
    // It's important to know how many frames per second are being rendered,
    // so I have some code to print this to debug output.
    //
    count++;
    if ( (count % 100 ) == 0 )
    {
        DWORD elapsedTime = GetTickCount() - startTime;
        float fps = (float) count / elapsedTime;
        fps *= 1000.0f;
        Trace( "%f frames per second\n", fps );
    }
    pGraphics->iterate();
    return true;
}
